﻿using System;
using System.ComponentModel;
using System.Threading;
using System.Threading.Tasks;
using Windows.Devices.I2c;
using Windows.UI.Core;
using Windows.UI.Xaml.Controls;

namespace I2c
{

    public sealed partial class MainPage : Page, INotifyPropertyChanged
    {
        public MainPage()
        {
            this.InitializeComponent();
        }

        private void Page_Loaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            this.DataContext=this;
            BeginAsync();
        }

        #region Property / INPC

        private double _temp;
        public double Temp
        {
            get { return _temp; }
            set
            {
                _temp = value;
                RaisePropertyChanged(nameof(Temp));
            }
        }
        private double _hum;
        public double Hum
        {
            get
            {
                return _hum;
            }
            set
            {
                _hum = value;
                RaisePropertyChanged(nameof(Hum));
            }
        }

        private void RaisePropertyChanged(string name)
        {
            if (PropertyChanged == null)
            {
                return;
            }
            PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion 
        
        Timer _timer;

        I2cDevice device;
        public async void BeginAsync()
        {
            byte address = 0x40;
            device = await InitI2cDevice(address);

            // デバイス初期化
            // 測定モードを温度湿度同時にする。
            device.Write(new byte[] { 0x02, 0x10, 0x00 });

            _timer = new Timer(MeasureTemp, null, 0, Timeout.Infinite);
        }

        private async void MeasureTemp(object state)
        {
            // 測定開始
            device.Write(new byte[] { 0x00 });
            // センサーの測定待ち。測定にに13[ms]かかるので余裕を持って100[ms]待つ。
            await Task.Delay(100);
            var buf = new byte[4];
            device.Read(buf);
            // 仕様書通りに変換
            var temp = (buf[0] * 256.0 + buf[1]) / 65536 * 165.0 - 40.0;
            var hum = (buf[0] * 256.0 + buf[1]) / 65536 * 100.0;

            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () => {
                this.Temp = temp;
                this.Hum = hum;
            });

            // 1秒毎に更新
            _timer.Change(1000, Timeout.Infinite);
        }

        private static async Task<I2cDevice> InitI2cDevice(byte address)
        {
            var aqs = I2cDevice.GetDeviceSelector("I2C1");
            var device_information_collection = await Windows.Devices.Enumeration.DeviceInformation.FindAllAsync(aqs);
            var deviceId = device_information_collection[0].Id;
            var connectionSetting = new I2cConnectionSettings(address);
            connectionSetting.BusSpeed = I2cBusSpeed.FastMode;
            connectionSetting.SharingMode = I2cSharingMode.Shared;

            return await I2cDevice.FromIdAsync(deviceId, connectionSetting);
        }
    }
}
